@@ -75,7 +75,82 @@ Select_model_aic = function(data_train, step){
 
 #-----------------------------------------------------------------------------------------------------------------------------------
 
-matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
+# forcer une variable
+
+matrices_MBV_lasso_parallel_forced = function (data, N, pi, numCores, indice){
+
+  # debuter l'enrigistrement
+  tic.clearlog()
+  tic()
+
+  p = dim(data)[2] - 1
+
+  #index de la variable forcer
+
+  # initialisation des différents matrices
+  matrice_M_hat = matrix(0, ncol = p, nrow = N)
+  matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
+  vect_intercept = rep(0, N)
+  #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
+
+  # Mise en place de la parrallélisation
+  cl = makePSOCKcluster(numCores)
+  clusterExport(cl, varlist = c("pi", "p", "N", "indice",  "data", "split", "Select_model_lasso"), envir = environment())
+  registerDoParallel(cl)
+
+  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("glmnet", "magrittr")) %dopar%{
+
+    s = split(data, pi)
+    data_train = s$train
+    data_test = s$test
+
+    # selection du model sur le train
+    s = Select_model_lasso(data_train)
+    M_hat = s$m # les indexs du model selectionnés
+
+    # forcer la variable
+    M_hat = c(indice, M_hat)
+    M_hat = M_hat%>%unique()
+
+
+    # data_select
+    X = data_test[, M_hat]
+    y = data_test[, dim(data_test)[2]]
+
+    # estimations des coéfficients
+    model = lm(y~., data = data.frame(X, y))
+
+    vect_coef = s$vect_coef #stocke les coéfficients sélectionnés
+    vect_coef[indice] = 1
+    # récupère les estimations
+    betaM_hat = model$coefficients[-1]
+    intercept = model$coefficients[1]
+    #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])^2
+
+
+
+    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, intercept = intercept, index = M_hat))
+
+  }
+  stopCluster(cl)
+
+  for (i in 1:N){
+    matrice_M_hat[i, ] = unlist(mat[i, "vect_coef"])
+    matriceBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "betaM_hat"])))
+    #matriceVarBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "VarBetaM_hat"])))
+  }
+  vect_intercept = unname(unlist(mat[, "intercept"]))
+
+  toc(log = TRUE, quiet = TRUE)
+  logs = tic.log()
+  time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
+  tic.clearlog()
+
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
+}
+
+
+matrices_MBV_aic_parallel_forced = function (data, N, pi,  numCores, indice, direction){
 
   # debuter l'enrigistrement
   tic.clearlog()
@@ -83,10 +158,81 @@ matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
 
   p = dim(data)[2] - 1
 
+  # initialisation des différents matrices
+  matrice_M_hat = matrix(0, ncol = p, nrow = N )
+  matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
+  vect_intercept = rep(0, N)
+
+  #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
+
+  # Mise en place de la parrallélisation
+  cl = makePSOCKcluster(numCores)
+  clusterExport(cl, varlist = c("pi", "p", "N", "indice", "data", "split", "Select_model_aic"), envir = environment())
+  registerDoParallel(cl)
+
+  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("MASS", "magrittr")) %dopar%{
+
+    s = split(data, pi)
+    data_train = s$train
+    data_test = s$test
+
+    # selection du model sur le train
+    select = Select_model_aic(data_train, step = direction)
+    M_hat  = select$m
+
+    #forcer la variable
+    M_hat = c(indice, M_hat)
+    M_hat = M_hat%>%unique()
+
+    vect_coef = select$vect_coef
+    vect_coef[indice] = 1
+
+    # data_select
+    X = data_test[, M_hat]
+    y = data_test[, dim(data_test)[2]]
+
+    # estimations des coéfficients
+    model = lm(y~., data = data.frame(X, y))
+    # récupère les estimations
+    betaM_hat = unname(model$coefficients[-1])
+    intercept = unname(model$coefficients[1])
+    #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])^2
+
+    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, intercept = intercept, index = M_hat))
+
+  }
+  stopCluster(cl)
+
+  for (i in 1:N){
+    matrice_M_hat[i, ] = unlist(mat[i, "vect_coef"])
+    matriceBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "betaM_hat"])))
+    #matriceVarBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "VarBetaM_hat"])))
+  }
+  vect_intercept = unname(unlist(mat[, "intercept"]))
+
+  toc(log = TRUE, quiet = TRUE)
+  logs = tic.log()
+  time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
+  tic.clearlog()
+
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
+}
+
+
+matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
+
+  # debuter l'enrigistrement
+  tic.clearlog()
+  tic()
+
+  p = dim(data)[2] - 1
+
+  #index de la variable forcer
+
   # initialisation des différents matrices
   matrice_M_hat = matrix(0, ncol = p, nrow = N)
   matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
-  #vect_intercept = rep(0, N)
+  vect_intercept = rep(0, N)
   #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
 
   # Mise en place de la parrallélisation
@@ -94,7 +240,8 @@ matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
   clusterExport(cl, varlist = c("pi", "p", "N", "data", "split", "Select_model_lasso"), envir = environment())
   registerDoParallel(cl)
 
-  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("glmnet", "tidyverse")) %dopar%{
+  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("glmnet", "magrittr")) %dopar%{
+
     s = split(data, pi)
     data_train = s$train
     data_test = s$test
@@ -114,12 +261,12 @@ matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
     vect_coef = s$vect_coef #stocke les coéfficients sélectionnés
     # récupère les estimations
     betaM_hat = model$coefficients[-1]
-    #intercept = model$coefficients[1]
+    intercept = model$coefficients[1]
     #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])^2
 
 
 
-    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, index = M_hat))
+    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, intercept = intercept, index = M_hat))
 
   }
   stopCluster(cl)
@@ -129,15 +276,16 @@ matrices_MBV_lasso_parallel = function (data, N, pi, numCores){
     matriceBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "betaM_hat"])))
     #matriceVarBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "VarBetaM_hat"])))
   }
-  #vect_intercept = unname(unlist(mat[, "intercept"]))
+  vect_intercept = unname(unlist(mat[, "intercept"]))
 
   toc(log = TRUE, quiet = TRUE)
   logs = tic.log()
   time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
   tic.clearlog()
 
-  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, var_names = colnames(data[, -dim(data)[2]]), time = time))
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
 }
+
 #-----------------------------------------------------------------------------------------
 
 matrices_MBV_aic_parallel = function (data, N, pi,  numCores, direction){
@@ -151,7 +299,7 @@ matrices_MBV_aic_parallel = function (data, N, pi,  numCores, direction){
   # initialisation des différents matrices
   matrice_M_hat = matrix(0, ncol = p, nrow = N )
   matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
-  #vect_intercept = rep(0, N)
+  vect_intercept = rep(0, N)
 
   #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
 
@@ -160,7 +308,7 @@ matrices_MBV_aic_parallel = function (data, N, pi,  numCores, direction){
   clusterExport(cl, varlist = c("pi", "p", "N", "data", "split", "Select_model_aic"), envir = environment())
   registerDoParallel(cl)
 
-  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("MASS, tidyverse")) %dopar%{
+  mat = foreach(1:N, .combine = rbind, .multicombine = TRUE, .packages = c("MASS", "magrittr")) %dopar%{
 
     s = split(data, pi)
     data_train = s$train
@@ -179,10 +327,10 @@ matrices_MBV_aic_parallel = function (data, N, pi,  numCores, direction){
     model = lm(y~., data = data.frame(X, y))
     # récupère les estimations
     betaM_hat = unname(model$coefficients[-1])
-    #intercept = unname(model$coefficients[1])
+    intercept = unname(model$coefficients[1])
     #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])^2
 
-    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, index = M_hat))
+    return(list(vect_coef = vect_coef, betaM_hat = betaM_hat, intercept = intercept, index = M_hat))
 
   }
   stopCluster(cl)
@@ -192,13 +340,14 @@ matrices_MBV_aic_parallel = function (data, N, pi,  numCores, direction){
     matriceBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "betaM_hat"])))
     #matriceVarBetaM_hat[i, unlist(mat[i, "index"])] = unname((unlist(mat[i, "VarBetaM_hat"])))
   }
+  vect_intercept = unname(unlist(mat[, "intercept"]))
 
   toc(log = TRUE, quiet = TRUE)
   logs = tic.log()
   time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
   tic.clearlog()
 
-  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, var_names = colnames(data[, -dim(data)[2]]), time = time))
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
 }
 
 #--------------------------------------------------------------------------------------------------------------------------------
@@ -213,7 +362,7 @@ matrices_MBV_lasso = function(data , N, pi){
   # initialisation des différents matrices
   matrice_M_hat = matrix(0, ncol = p, nrow = N)
   matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
-  #vect_intercept = rep(0, N)
+  vect_intercept = rep(0, N)
   #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
 
   for (i in 1:N){
@@ -235,7 +384,7 @@ matrices_MBV_lasso = function(data , N, pi){
 
     # récupère les estimations
     betaM_hat = model$coefficients[-1]
-    #vect_intercept[i] = model$coefficients[1]
+    vect_intercept[i] = model$coefficients[1]
     #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])
 
     # stockage dans les matrices correspondantes
@@ -248,7 +397,61 @@ matrices_MBV_lasso = function(data , N, pi){
   time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
   tic.clearlog()
 
-  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, var_names = colnames(data[, -dim(data)[2]]), time = time))
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
+}
+
+matrices_MBV_lasso_forced = function(data , N, pi, indice){
+
+  # debuter l'enrigistrement
+  tic.clearlog()
+  tic()
+
+  p = dim(data)[2] - 1
+  # initialisation des différents matrices
+  matrice_M_hat = matrix(0, ncol = p, nrow = N)
+  matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
+  vect_intercept = rep(0, N)
+  #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
+
+  for (i in 1:N){
+    s = split(data, pi)
+    data_train = s$train
+    data_test = s$test
+
+    # selection du model sur le train
+    s = Select_model_lasso(data_train)
+    M_hat = s$m
+    M_hat = c(indice, M_hat)
+    M_hat = M_hat%>%unique()
+
+    #s$vect_coef [M_hat] = 1
+    vect_coef = s$vect_coef
+    vect_coef[indice] = 1
+    matrice_M_hat[i, ] = vect_coef #stocke les coéfficients sélectionnés
+
+    # data_select
+    X = data_test[, M_hat]
+    y = data_test[, dim(data_test)[2]]
+
+    # estimations des coéfficients
+    model = lm(y~., data = data.frame(X, y))
+
+    # récupère les estimations
+    betaM_hat = model$coefficients[-1]
+    vect_intercept[i] = model$coefficients[1]
+    #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])
+
+    # stockage dans les matrices correspondantes
+    matriceBetaM_hat[i, M_hat] = betaM_hat
+    #matriceVarBetaM_hat[i, M_hat] = VarBetaM_hat
+  }
+
+  toc(log = TRUE, quiet = TRUE)
+  logs = tic.log()
+  time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
+  tic.clearlog()
+
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
 }
 
 matrices_MBV_aic = function(data , N, pi, direction){
@@ -261,7 +464,7 @@ matrices_MBV_aic = function(data , N, pi, direction){
   # initialisation des différents matrices
   matrice_M_hat = matrix(0, ncol = p, nrow = N)
   matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
-  #vect_intercept = rep(0, N)
+  vect_intercept = rep(0, N)
   #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
 
   for (i in 1:N){
@@ -283,7 +486,7 @@ matrices_MBV_aic = function(data , N, pi, direction){
 
     # récupère les estimations
     betaM_hat = model$coefficients[-1]
-    #vect_intercept[i] = model$coefficients[1]
+    vect_intercept[i] = model$coefficients[1]
     #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])
 
     # stockage dans les matrices correspondantes
@@ -296,9 +499,61 @@ matrices_MBV_aic = function(data , N, pi, direction){
   time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
   tic.clearlog()
 
-  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, var_names = colnames(data[, -dim(data)[2]]), time = time))
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
 }
 
+matrices_MBV_aic_forced = function(data , N, pi, indice, direction){
+
+  # debuter l'enrigistrement
+  tic.clearlog()
+  tic()
+
+  p = dim(data)[2] - 1
+  # initialisation des différents matrices
+  matrice_M_hat = matrix(0, ncol = p, nrow = N)
+  matriceBetaM_hat = matrix(0, ncol = p, nrow = N)
+  vect_intercept = rep(0, N)
+  #matriceVarBetaM_hat = matrix(0, ncol = p, nrow = N)
+
+  for (i in 1:N){
+    s = split(data, pi)
+    data_train = s$train
+    data_test = s$test
+
+    # selection du model sur le train
+    s = Select_model_aic(data_train, step = direction)
+    M_hat = s$m
+    M_hat = c(indice, M_hat)
+    M_hat = M_hat%>%unique()
+
+    vect_coef = s$vect_coef
+    vect_coef[indice] = 1
+    matrice_M_hat[i, ] = vect_coef #stocke les coéfficients sélectionnés
+
+    # data_select
+    X = data_test[, M_hat]
+    y = data_test[, dim(data_test)[2]]
+
+    # estimations des coéfficients
+    model = lm(y~., data = data.frame(X, y))
+
+    # récupère les estimations
+    betaM_hat = model$coefficients[-1]
+    vect_intercept[i] = model$coefficients[1]
+    #VarBetaM_hat = unname(summary(model)$coefficients[, "Std. Error"][-1])
+
+    # stockage dans les matrices correspondantes
+    matriceBetaM_hat[i, M_hat] = betaM_hat
+    #matriceVarBetaM_hat[i, M_hat] = VarBetaM_hat
+  }
+
+  toc(log = TRUE, quiet = TRUE)
+  logs = tic.log()
+  time = as.numeric(gsub(" sec elapsed", "", logs[[1]]))
+  tic.clearlog()
+
+  return(list(m_M = matrice_M_hat, m_beta = matriceBetaM_hat, vect_intercept = vect_intercept, var_names = colnames(data[, -dim(data)[2]]), time = time))
+}
 #-----------------------------------------------------------------------------------------------------------------
 
 vote2 = function(matrices, N, emp_alpha){
@@ -325,6 +580,7 @@ vote2 = function(matrices, N, emp_alpha){
   BM2_hat = matrices$m_beta[indice_k2, ]#M2_vote]
   #VarBeta_hat = matrices$m_Var[indice_k2, ]#M2_vote]
 
+  intercept = matrices$vect_intercept[indice_k2]%>%mean() # intercept sur les lignes correspondant au modèle le plus fréquent
 
   # IC_empirique
   matrice_icEmp = data.frame(matrix(0, ncol = p, nrow = 2))
@@ -366,7 +622,7 @@ vote2 = function(matrices, N, emp_alpha){
   ic = data.frame(matrice_icEmp)
   colnames(ic) = matrices$var_names
 
-  return(list(gg = gg, m = M2_vote, beta = beta2, ic = ic, names_vars = matrices$var_names))
+  return(list(gg = gg, m = M2_vote, beta = beta2, b0 = intercept, ic = ic, names_vars = matrices$var_names))
 }
 
 vote3= function(matrices, N, emp_alpha, s){
@@ -375,7 +631,7 @@ vote3= function(matrices, N, emp_alpha, s){
 
   M = matrices$m_M
   M_beta = matrices$m_beta
-  M_var = matrices$m_Var
+  #M_var = matrices$m_Var
 
   M_vote = apply(M, MARGIN = 2, FUN = mean) > s
 
@@ -386,14 +642,26 @@ vote3= function(matrices, N, emp_alpha, s){
   matrice_icEmp = matrix(0, ncol = p, nrow = 2)
   beta = rep(0, p)
   #var = rep(0, p)
+  list = list() # initialiser un index_ligne qui va enrigistrer les index_ligne de chaque variable
   for (j in indice_j){
     #ligne non nulle
     index_ligne = which(M[,j]==1)
+    list[[j]] = index_ligne
     beta[j] = mean(M_beta[index_ligne,j]) # récupèrer beta chapeau
     #var[j] = mean(M_var[index_ligne, j]) # recuperer les var chapeau
     matrice_icEmp[, j] = unname(quantile(M_beta[index_ligne, j], c(emp_alpha/2, 1- (emp_alpha/2)))) # IC empirique
   }
 
+  # gerer l'intercept
+  list = list[!sapply(list, is.null)] # enlever les index NULL
+  names(list) = seq_along(list) # re-indexer la list
+  vect_long = sapply(list, length)
+  min_vect = list[[which.min(vect_long)]]
+
+  intercept = matrices$vect_intercept[min_vect]%>%mean()
+
+  # Comme decider de l'intercept ici???????
+
 
 
   # IC comme dans le vote1&2
@@ -432,28 +700,24 @@ vote3= function(matrices, N, emp_alpha, s){
   colnames(ic) = matrices$var_names
 
 
-  return(list(gg = gg, m = M3_vote, beta = beta , ic = ic, names_vars = matrices$var_names))
+  return(list(gg = gg, m = M3_vote, beta = beta , b0 = intercept, ic = ic, names_vars = matrices$var_names))
 }
 
 
-#--------------------------------------Mise en place des fonction du Packages --------------------------------------------------------------------------
-
 #' Function that handles storing our estimation and variable selection matrices during the different splits.
 #'
-#'
 #' @param method Method for variable selection. Should be one of \code{"Lasso"} or \code{"BIC"}.
 #' @param cores Number of cores for parallel processing.
 #' @param data Data set to be used for regression modeling.
 #' @param formula Regression model to use, specified as a formula.
-#' @param N Number of splittings.
-#' @param p_split Probabilities associated with the splittings.
-#' @param direction It can take two values: \code{"backward"} and \code{"forward"} in the case of BIC, it specifies the direction in which the selection will be made.
-#'
+#' @param N Number of splits.
+#' @param p_split Probabilities associated with the splits.
+#' @param forced_var A character string specifying a predictor variable to be forced into selection. By default, it is NULL, allowing for no forced selection. If provided, this variable will be consistently selected during the N splits.
+#' @param direction It can take two values: \code{"backward"} and \code{"forward"}. In the case of BIC, it specifies the direction in which the selection will be made.
 #'
 #' @details
 #' We have data that we will split several times while shuffling it each time. Then, we will divide the data into two parts based on a specific probability for splitting. In the first half, we will perform model selection, followed by calibration on the second half. At the end of these steps, we will obtain matrices of dimensions N*p that represent the selected models and the estimated coefficients associated with these models.
 #'
-#'
 #' @importFrom stats coef lm quantile formula
 #' @importFrom MASS stepAIC
 #' @importFrom glmnet cv.glmnet glmnet
@@ -464,96 +728,204 @@ vote3= function(matrices, N, emp_alpha, s){
 #' @importFrom foreach foreach %dopar%
 #' @import mlbench
 #' @import tictoc
-#' @return Un objet de classe lmps
+#' @return An object of class lmps
 #'
-#'@examples
+#' @examples
 #'
 #' library(mlbench)
 #' data("BostonHousing")
 #' # lmps object
 #' model = lmps(medv ~ ., data = BostonHousing, method = "Lasso", N = 50)
 #'
-#'
 #' \donttest{
-#  #A parallelized example
+#' # A parallelized example
 #' # lmps object
 #' model = lmps(medv ~ ., data = BostonHousing, method = "Lasso", N = 50, cores = 2)
-#'
-#'
-#'
 #' }
 #'
-#'
-#'@export
-lmps = function(formula, data, method , N, p_split = 0.5, cores = NULL, direction = "backward"){
-  # method = c("Lasso", "BIC")
-  # vote = c("model", "coef")
-  # numCores : = nombre de coeurs pour la parallélisation
-  # s.vect_coef := si jamais on utilise vote sur les coefficients, la proba de selection d'une variable
-  # data := data set
-  # formula : = model de regression
-  # N := nombres de splittages
-  # p_split := probabiltés de splittages
-
-  #### pour la méthode
+#' @export
 
 
-
-
-  if (method%in%c("Lasso", "BIC")==FALSE){
+lmps = function(formula, data, method, N, p_split = 0.5, cores = NULL, direction = "backward", forced_var = NULL) {
+  # Vérification de la méthode
+  if (!method %in% c("Lasso", "BIC")) {
     stop("Error on method argument, check ??lmps")
   }
 
-
-
   ##################  Regler la formule ##################################
   variables = all.vars(formula)
-
   target = variables[1]
 
   if ("." %in% all.names(formula)) {
     predictors = setdiff(names(data), target)
   } else {
-
     predictors = variables[-1]
   }
 
-  data = data[, c(predictors, target)]
+  data = data[, c(predictors, target), drop = FALSE]  # Assurez-vous que data reste un data.frame
 
-
-  ################# Regler la parrallélisation ###################
-  if(is.null(cores)){
-
-    # post_selection
-    if (method == "BIC"){
-        matrices = matrices_MBV_aic(data, N, pi = p_split, direction = direction)
+  ######### Regler le problème forced_var ################
+  if (!is.null(forced_var)) {
+    if (!forced_var %in% predictors) {
+      stop("Error: Enter a valid variable name for forced_var")
     }
-    else{
-      matrices = matrices_MBV_lasso(data, N, pi = p_split)
-    }
-
+    indice = which(predictors == forced_var)
   }
 
-  else{
-    # post_selection
-    if (method == "BIC"){
-      matrices = matrices_MBV_aic_parallel(data, N, pi = p_split, numCores = cores, direction = direction)
+  ################# Regler la parallélisation ###################
+  if (is.null(cores)) {
+    if (method == "BIC") {
+      matrices = if (is.null(forced_var)) {
+        matrices_MBV_aic(data, N, pi = p_split, direction = direction)
+      } else {
+        matrices_MBV_aic_forced(data, N, pi = p_split, indice = indice, direction = direction)
+      }
+    } else {
+      matrices = if (is.null(forced_var)) {
+        matrices_MBV_lasso(data, N, pi = p_split)
+      } else {
+        matrices_MBV_lasso_forced(data, N, pi = p_split, indice = indice)
+      }
     }
-    else{
-      matrices = matrices_MBV_lasso_parallel(data, N, pi = p_split, numCores = cores)
+  } else {
+    if (method == "BIC") {
+      matrices = if (is.null(forced_var)) {
+        matrices_MBV_aic_parallel(data, N, pi = p_split, numCores = cores, direction = direction)
+      } else {
+        matrices_MBV_aic_parallel_forced(data, N, pi = p_split, numCores = cores, indice = indice, direction = direction)
+      }
+    } else {
+      matrices = if (is.null(forced_var)) {
+        matrices_MBV_lasso_parallel(data, N, pi = p_split, numCores = cores)
+      } else {
+        matrices_MBV_lasso_parallel_forced(data, N, pi = p_split, indice = indice, numCores = cores)
+      }
     }
-
-
   }
 
-  lmps =  list(nb_splittings = N, matrix = matrices)
+  # Créer la liste de résultats
+  lmps = list(nb_splittings = N, matrix = matrices)
   class(lmps) = "lmps"
 
-
   return(invisible(lmps))
 }
 
 
+# lmps = function(formula, data, method , N, p_split = 0.5, cores = NULL, direction = "backward", forced_var = NULL){
+#   # method = c("Lasso", "BIC")
+#   # vote = c("model", "coef")
+#   # numCores : = nombre de coeurs pour la parallélisation
+#   # s.vect_coef := si jamais on utilise vote sur les coefficients, la proba de selection d'une variable
+#   # data := data set
+#   # formula : = model de regression
+#   # N := nombres de splittages
+#   # p_split := probabiltés de splittages
+#
+#   #### pour la méthode
+#
+#
+#
+#
+#   if (method%in%c("Lasso", "BIC")==FALSE){
+#     stop("Error on method argument, check ??lmps")
+#   }
+#
+#
+#
+#   ##################  Regler la formule ##################################
+#   variables = all.vars(formula)
+#
+#   target = variables[1]
+#
+#   if ("." %in% all.names(formula)) {
+#     predictors = setdiff(names(data), target)
+#   } else {
+#
+#     predictors = variables[-1]
+#   }
+#
+#   data = data[, c(predictors, target)]
+#
+#   ######### Regler le problème forced_var
+#
+#   if(is.null(forced_var)){
+#
+#     ################# Regler la parrallélisation ###################
+#     if(is.null(cores)){
+#
+#       # post_selection
+#       if (method == "BIC"){
+#         matrices = matrices_MBV_aic(data, N, pi = p_split, direction = direction)
+#       }
+#       else{
+#         matrices = matrices_MBV_lasso(data, N, pi = p_split)
+#       }
+#
+#     }
+#
+#     else{
+#       # post_selection
+#       if (method == "BIC"){
+#         matrices = matrices_MBV_aic_parallel(data, N, pi = p_split, numCores = cores, direction = direction)
+#       }
+#       else{
+#         matrices = matrices_MBV_lasso_parallel(data, N, pi = p_split, numCores = cores)
+#       }
+#
+#
+#     }
+#
+#   }
+#
+#   else{
+#
+#     if(forced_var %in% predictors){
+#       indice = which(predictors == forced_var)
+#
+#       ################# Regler la parrallélisation ###################
+#       if(is.null(cores)){
+#
+#         # post_selection
+#         if (method == "BIC"){
+#           matrices = matrices_MBV_aic_forced(data, N, pi = p_split, indice = indice, direction = direction)
+#         }
+#         else{
+#           matrices = matrices_MBV_lasso_forced(data, N, pi = p_split, indice = indice)
+#         }
+#
+#       }
+#
+#       else{
+#         # post_selection
+#         if (method == "BIC"){
+#           matrices = matrices_MBV_aic_parallel_forced(data, N, pi = p_split, numCores = cores, indice = indice, direction = direction)
+#         }
+#         else{
+#           matrices = matrices_MBV_lasso_parallel_forced(data, N, pi = p_split, indice = indice, numCores = cores)
+#         }
+#
+#       }
+#
+#
+#
+#     }
+#
+#     else{
+#       stop("Error: Entrer un nom de variable à forced valide")
+#     }
+#
+#   }
+#
+#
+#
+#   lmps =  list(nb_splittings = N, matrix = matrices)
+#   class(lmps) = "lmps"
+#
+#
+#   return(invisible(lmps))
+# }
+
+
 
 # function summary
 
@@ -653,6 +1025,7 @@ summary.lmps = function(object, ...){
 #' this function allows us to compute confidence intervals that are calculated empirically based on the chosen voting method and the desired level of certainty.
 #' The confidence intervals are obtained through empirical calculation on each vector of estimates for the corresponding coefficients.
 #'
+#'CIps also provides an intercept (test version) estimated as follows: in the case of a vote on models, it takes the average of the intercept vector for the rows where the most frequently selected model in the N splits is chosen. For the vote on coefficients, the idea is to select the coefficient that has been chosen the least number of times among those retained and then average the intercept only for the rows where this coefficient is selected.
 #'
 #'@examples
 #' library(mlbench)
@@ -787,6 +1160,57 @@ plot.CIps = function(x, ...){
 }
 
 
+#' A predict function for cips
+#'
+#' This function generates predictions based on a cips object.
+#'
+#' @param object An object of class 'cips'.
+#' @param newdata A dataframe containing new data to make predictions.
+#' @param X Explanatory variables, default is NULL.
+#' @param y Target corresponding to the explanatory variables, default is NULL.
+#' @param ... Additional arguments for future use.
+#'
+#' @details When X and y are not NULL, they represent the data used to select the model.
+#' These data are then reused by recalibrating on the selected subset to obtain the beta estimates (hybrid approach).
+#' @return A numeric vector of predicted values.
+#' @export
+predict.CIps = function(object, newdata, X = NULL, y = NULL,  ...){
+
+
+  if(dim(newdata)[2]!= length(object$names_vars)){
+    stop("Error: different variable numbers")
+  }
+
+  # réarranger le data test comme names_vars
+  X_test = newdata[, object$names_vars]
+  X_test = X_test%>%as.matrix(ncol = length(object$numvars))%>%apply(MARGIN = 2, FUN = as.numeric)
+
+  if(is.null(X)){
+
+    pred = X_test%>%as.matrix()%*% as.matrix(object$beta) + object$b0
+  }
+
+  else{
+
+    dim = dim(X)[2] #enrigistrer la dimension
+    X = X[, object$names_vars] # réordonner au cas où
+    vars_select = object$m%>%as.integer()
+    X = X[, which(vars_select!=0)]
+    data = data.frame(X = X, y = y)
+    colnames(data) = c(paste0("x_", (1:dim(X)[2])), "y")
+    model = lm(y~., data = data)
+    b0 = model$coefficients[1]
+    beta = rep(0, dim)
+    beta [which(vars_select!=0)] = model$coefficients[-1]
+
+    pred = X_test%>%as.matrix()%*%as.matrix(beta) + b0
+
+  }
+
+
+  return (pred)
+}
+
 
 
 
